/* ***************************************************************************************************
** 	Purpose: To automatically generate the demographic distribution table comparing two groups
**	
**	Required inputs:
**   		 1). input_dir:  	CSV input file directory  
**			 2). output_dir: 	Output file directory 
**			 3). input_file: 	Independent variable list input file with only three columns
**			 4). output_file: 	Output CSV file name
**			 5). grpvar: 		Group variable
**			 6). srcdata: 		Source SAS analytic file name
**			 7). varlevel_ge: 	To control the export file whether we need the baseline row for dummy variable
**			 8). cohort: 		Control the data cohort to do the analysis
**
**	Author: Zhaohui Fan
**
** 	Affiliation: CHOP @ University of Michigan
** 
**	Last Updated: 10/30/2022
**
**************************************************************************************************** */

options compress=YES source source2 msglevel=I SYMBOLGEN;

%macro demotb(    input_dir = 		/* CSV input file directory  */
				, output_dir = 		/* Output file directory  */
				, input_file = 		/* Independent variable list input file with only three columns */
				, output_file = 	/* Output CSV file name */
				, grpvar = 			/* Group variable */
				, srcdata = 		/* Soruce SAS analytic file name*/
				, varlevel_ge = 	/* To control the export file whether we need the baseline row for dummy variable */
				, cohort = 			/* Control the data cohort (rows) to do the analysis */
			  );


libname out "&output_dir";	/* Source analytic file directory lib reference */

	/* 1. Read in the independent variable list from CSV file */
	    data indepvar_list;
	        infile "&input_dir.&input_file" dlm=',' dsd firstobs=2 truncover lrecl=1000 termstr=CRLF;
	        input var_order		: 2.
	              var_name		: $32.
				  var_type		: $3.
	        ;
		proc sort;
		by var_order;
	    run;

	/* 2. Read in the source analytic table */
		 data workdata;
		 set out.&srcdata;
		 	&cohort;
		 run;

	/* 3. Define the dynamic macro variables for categorical independent variables */
		proc sql;
			select put(count(var_name),8.-L)  into :catcnt
			from indepvar_list
			where var_type="cat"
			order by var_order;  
				 
			select var_name into :catvar1-:catvar&catcnt 
			from indepvar_list
			where var_type="cat"
			order by var_order; 

			select var_order into :catorder1-:catorder&catcnt 
			from indepvar_list
			where var_type="cat"
			order by var_order; 
		quit;

	/* 4. Loop through each categorical independent variable */
	%do i = 1 %to &catcnt;

		/* 4a. Counts & Frequency */
		proc sql;
		create table cat_&i as 
			select &&catorder&i as var_order
				 , "&&catvar&i" as var_name
				 , &&catvar&i as var_level
				 , compress(yes_cnt)||" ("||compress(yes_pct)||"%)" as y_stats
				 , compress(no_cnt)||" ("||compress(no_pct)||"%)" as n_stats
			from(select  y.&&catvar&i
						, put(y_cnt, 10.0) as yes_cnt
						, put(y_cnt/yt_cnt*100,10.1) as yes_pct
						, put(n_cnt, 10.0) as no_cnt
						, put(n_cnt/nt_cnt*100,10.1) as no_pct
				 from (select  &&catvar&i, count(&&catvar&i) as y_cnt from workdata where &grpvar=1 group by &&catvar&i) y
						left join 
					  (select  &&catvar&i, count(&&catvar&i) as n_cnt from workdata where &grpvar=0 group by &&catvar&i) n
					   on y.&&catvar&i =n.&&catvar&i
				 cross join (select count(&&catvar&i) as yt_cnt from workdata where &grpvar=1) yt
				 cross join (select count(&&catvar&i) as nt_cnt from workdata where &grpvar=0) nt
				);
		quit;

		/* 4b. P-value from Chisq test */
		proc freq data=workdata;
		tables &&catvar&i*&grpvar/chisq;
		ods output ChiSq=chisq_&i (where = (statistic = 'Chi-Square') keep=table Prob statistic);
		run;
		
		/* 4c. Merge P-value to the count table */
		proc sql;
			create table cat_p&i as
			select    d.* 
					, c.prob as p_value /*format=$8.*/
				from cat_&i d
			cross join chisq_&i c
			order by var_order, var_level;
		quit;

	%end;
	
	/* 5. Vertically stack the tables together from each categorical independent variable */
	data cat_p;
		length var_order 8 var_name $32 var_level 8 y_stats $32 n_stats $32;
		set cat_p1-cat_p&catcnt;
	run;

	/* 6. Define the dynamic macro variables for continuous independent variables */
		proc sql;
			select put(count(var_name),8.-L)  into :concnt
			from indepvar_list
			where var_type="con"
			order by var_order;  
				 
			select var_name into :convar1-:convar&concnt 
			from indepvar_list
			where var_type="con"
			order by var_order; 
				 
			select var_order into :conorder1-:conorder&concnt 
			from indepvar_list
			where var_type="con"
			order by var_order; 

		quit;

/* If no continuous indepentent variables, then skip the below section */
/***********************************************************************************/
		%if &concnt = 0 %then %goto skipcon;

			/* 7. Loop through each continuous independent variable */
			%do i = 1 %to &concnt;

				/* 7a. t-test */

				proc ttest data=workdata;
				class &grpvar;
				var &&convar&i;
				ods output Statistics=stats_&i TTests=ttest_&i;
				run;

				/* 7b. Get the mean and stddev*/
				proc sql;
					create table con_p&i as 
					select &&conorder&i as var_order
						, m.variable as var_name
						, 1 as var_level
						, y_stats
						, n_stats
						, probt as p_value
					from (select variable
							, max(y_stats) as y_stats
							, max(n_stats) as n_stats
						  from(select compress(variable) as variable
								, case when class = "1" then compress(put(mean, 10.2)) ||"  "||compress(put(stddev, 10.2)) end as y_stats
								, case when class = "0" then compress(put(mean, 10.2)) ||"  "||compress(put(stddev, 10.2)) end as n_stats
								from stats_&i
							) group by m.variable
						) m
					left join (select compress(variable) as variable, probt from ttest_&i where method="Pooled") p on m.variable = p.variable;

			quit;
			%end;

			/* 8. Vertically stack the tables together from each continuous independent variable */
			data con_p;
				length var_order 8 var_name $32 var_level 8 y_stats $32 n_stats $32;
				set con_p1-con_p&concnt;
			run;
	%skipcon:;
/*******************************************************************************************************/		   
		/* 9. Vertically stack the categorical and continuous (if have) table together and sort by variable order */
		proc sql;
			create table temp_cat_con_p as
			select * from(	select * from cat_p
							%if &concnt > 0 %then %do;
								union all
								select * from con_p
							%end;
						 )
			order by var_order, var_level;
		quit;

		data cat_con_p;
			length var_order 8 var_name $32 var_level 8 y_stats $32 n_stats $32;
			set temp_cat_con_p;
			by var_order var_level;
			if not last.var_order then do;
 			var_order="";
			var_name="";
			p_value="";
			end;
		run;

		/* 10. Export to a csv file */
		PROC EXPORT DATA= cat_con_p (where=(var_level>=&varlevel_ge))
		            OUTFILE= "&output_dir.&output_file" 
		            DBMS=CSV REPLACE;
		     PUTNAMES=YES;
		RUN;

%mend;
